#ifndef _NEW_DB_CPP
#define _NEW_DB_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SQLExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedClasses/CRC32/CRC.H"
#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../../SharedClasses/CGetPKs/CGetPKs.H"
#include "../../SharedSource/SQLTemplate.H"
#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedClasses/CStatusDlg/CStatusDlg.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "Command.H"
#include "Console.H"
#include "SQLImport.H"
#include "NewDB.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SetupNewClient(char *sClientDBFileName, char *sDBO, char *sDBName, bool bImportStructure, 
					bool bImportData, bool bDropImportTable, CStatusDlg *lpDlg)
{
	/*
		sClientDBFileName: Contains the name of the backup file that contains the users existing
			data and structure. (Relative to the server)

		sDBName: Contains the base name of the new DB to create for the new client.

		bImportStructure: Should we import the structure of the sClientDBFileName to our new DB?

		bImportData: Should we import the data from the sClientDBFileName to our new DB?
	*/

    CSQL MySQL;

	char sSQL[1024];

	if(bImportData && !bImportStructure)
	{
		lpDlg->SetText("You cannot import data if you opt not to inport the DB structure.");
		return false;
	}

	lpDlg->SetText("Connecting to the SQL server.");
	if(!MySQL.Connect(gsSQLCustDriver, gsSQLCustServer, gsSQLCustUserID, gsSQLCustPassword, "Master"))
    {
        lpDlg->SetText("Error connecting to the [Master] database.");
        return false;
    }

	if(bImportStructure || bImportData)
	{
		lpDlg->SetText("Restoring the import database. (This could take some time)");
		if(!RestoreDBForImport(&MySQL, sClientDBFileName, sDBName))
		{
			lpDlg->SetText("Failed to restore the clients database.");
			MySQL.Disconnect();
			return false;
		}
	}

	lpDlg->SetText("Creating the new client database.");
	if(!CreateNewDB(&MySQL, sDBName))
	{
		lpDlg->SetText("Failed to create the new database.");
		MySQL.Disconnect();
		return false;
	}


	if(bImportStructure || bImportData)
	{
		lpDlg->SetText("Importing the database structure.");
		if(!ImportDBStructure(&MySQL, sDBO, sDBName, bImportData, lpDlg))
		{
			lpDlg->SetText("Failed to import the database structure.");
			MySQL.Disconnect();
			return false;
		}
	}

	if(bDropImportTable)
	{
		lpDlg->SetText("Dropping the import database.");
		sprintf(sSQL, "DROP DATABASE SQLExch_%s_IMP_Temp", sDBName);
		MySQL.ExecuteNonQuery(sSQL);
	}

	MySQL.Disconnect();

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ImportDBStructure(CSQL *lpMySQL, char *sDBO, char *sDBName, bool bImportData, CStatusDlg *lpDlg)
{
	char *sBaseSQL = NULL;
	char sSQL[5120];

	int iTempSz = 0;

	FILE *hSource = NULL;
	
	char *sLargeSQL = NULL;
	char sTable[1024];
	long lTableID = 0;

	char sColName[1024];
	char sColType[255];
	long lColLen = 0;
	long lColStatus = 0;
	long lColNullable = 0;

	char sSmlTmp[32];
	char sLgTmp[1024];

	char sPKName[255];
	int iPK = 0;
	int iRow = 0;

	CGetPKs cMyPKs;

	CRecordSet rsTables;
	CRecordSet rsColumns;

	sBaseSQL = "SELECT Name, ID FROM SQLExch_%s_IMP_Temp.%s.SysObjects"
		" WHERE xType = 'U' AND Name <> 'dtProperties' AND LEFT(Name, 8) <> 'SQLExch_'"
		" ORDER BY Name";
	sprintf(sSQL, sBaseSQL, sDBName, gsDefaultDBO);

	if(!lpMySQL->Execute(sSQL, &rsTables))
	{
		lpMySQL->GetErrorMessage(sLgTmp, sizeof(sLgTmp), rsTables.hSTMT);
		MsgBox(sLgTmp);
		return false;
	}

	lpDlg->SetProgressRange(0, rsTables.RowCount);
	
	sLargeSQL = (char *) calloc(1024 * 1024, sizeof(char));
	Assert(!sLargeSQL, "Memory Allocation Error.");

	while(rsTables.Fetch())
	{
		rsTables.sColumnEx(1, sTable, sizeof(sTable), &iTempSz);
		lTableID = rsTables.lColumn(2);

		sBaseSQL = "SELECT xSysColumns.Name, xSysColumns.Length, xSysTypes.Name, xSysTypes.Status, xSysColumns.IsNullable"
			" FROM SQLExch_%s_IMP_Temp.[%s].SysColumns as xSysColumns, SQLExch_%s_IMP_Temp.[%s].SysTypes as xSysTypes"
			" WHERE ID = %d AND xSysColumns.xType = xSysTypes.xUserType"
			" ORDER BY ColOrder";
		sprintf(sSQL, sBaseSQL, sDBName, sDBO, sDBName, sDBO, lTableID);

		if(!lpMySQL->Execute(sSQL, &rsColumns))
		{
			lpMySQL->GetErrorMessage(sLgTmp, sizeof(sLgTmp), rsColumns.hSTMT);
			MsgBox(sLgTmp);

			rsTables.Close();
			free(sLargeSQL);
			return false;
		}

		//------------------------------------------------------------------------------------------
		//Drop the existing table
		//------------------------------------------------------------------------------------------
		lpMySQL->bThrowErrors = false;
		sprintf(sSQL, "DROP TABLE [SQLExch_%s].[%s].[%s]", sDBName, sDBO, sTable);
		lpMySQL->ExecuteNonQuery(sSQL);
		lpMySQL->bThrowErrors = true;
		//------------------------------------------------------------------------------------------

		sprintf(sLargeSQL, "CREATE TABLE [SQLExch_%s].[%s].[%s] (\r\n", sDBName, sDBO, sTable);

		sprintf(sLgTmp, "SQLExch_%s_IMP_Temp", sDBName);
		cMyPKs.Get(lpMySQL, sTable, sDBO, sLgTmp);

		while(rsColumns.Fetch())
		{
			rsColumns.sColumnEx(1, sColName, sizeof(sColName), &iTempSz);
			lColLen = rsColumns.lColumn(2);
			rsColumns.sColumnEx(3, sColType, sizeof(sColType), &iTempSz);
			lColStatus = rsColumns.lColumn(4);
			lColNullable = rsColumns.lColumn(5);

			strcat(sLargeSQL, "\t[");
			strcat(sLargeSQL, sColName);
			strcat(sLargeSQL, "] [");
			strcat(sLargeSQL, sColType);
			strcat(sLargeSQL, "]");
			if(lColStatus != 0)
			{
				sprintf(sSmlTmp, " (%d)", lColLen);
				strcat(sLargeSQL, sSmlTmp);
			}
			
			if(lColNullable == 0)
			{
				strcat(sLargeSQL, " NOT NULL,\r\n");
			}
			else {
				strcat(sLargeSQL, " NULL,\r\n");
			}
		}

		iTempSz = strlen(sLargeSQL);
		sLargeSQL[iTempSz - 1] = '\0';
		sLargeSQL[iTempSz - 2] = '\0';
		sLargeSQL[iTempSz - 3] = '\0';
		strcat(sLargeSQL, "\r\n");

		if(cMyPKs.iPKs > 0)
		{
			strcat(sLargeSQL, "\tCONSTRAINT [");
			strcat(sLargeSQL, sPKName);
			strcat(sLargeSQL, "] PRIMARY KEY CLUSTERED\r\n");
			strcat(sLargeSQL, "\t(\r\n");

			iPK = 0;
			while(iPK < cMyPKs.iPKs)
			{
				strcat(sLargeSQL, "\t\t[");
				strcat(sLargeSQL, cMyPKs.sPKs[iPK]);
				strcat(sLargeSQL, "]");

				if(iPK != (cMyPKs.iPKs - 1))
				{
					strcat(sLargeSQL, ",\r\n");
				}
				iPK++;
			}
			strcat(sLargeSQL, "\r\n\t) ON [PRIMARY]\r\n");
			
		}

		strcat(sLargeSQL, ") ON [PRIMARY]");

		//MsgBox(sLargeSQL);
		lpMySQL->ExecuteNonQuery(sLargeSQL);

		if(bImportData)
		{
			//------------------------------------------------------------------------------------------
			//Import the data.
			//------------------------------------------------------------------------------------------
			sprintf(sSQL, "INSERT INTO [SQLExch_%s].[%s].[%s] SELECT * FROM [SQLExch_%s_IMP_Temp].[%s].[%s]",
				sDBName, sDBO, sTable, sDBName, sDBO, sTable);
			lpMySQL->ExecuteNonQuery(sSQL);
			//------------------------------------------------------------------------------------------
		}

		cMyPKs.Free();
		rsColumns.Close();

		iRow++;

		lpDlg->SetProgressPos(iRow);
	}
	
	free(sLargeSQL);
	rsTables.Close();

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool RestoreDBForImport(CSQL *lpMySQL, char *sClientDBFileName, char *sDBName)
{
	char *sBaseSQL = NULL;
	char sSQL[5120];

	int iTempSz = 0;
	char sTemp1[1024];
	char sTemp2[1024];

	char sLogicalData[1024];
	char sLogicalLog[1024];

	CRecordSet rsTemp;

	sprintf(sSQL, "RESTORE FILELISTONLY FROM DISK = '%s'", sClientDBFileName);
	if(!lpMySQL->Execute(sSQL, &rsTemp))
	{
		MsgBox("The backup file specified is either invalid or does not exist.");
		return false;
	}

	strcpy(sLogicalData, "");
	strcpy(sLogicalLog, "");

	while(rsTemp.Fetch())
	{
		rsTemp.sColumnEx(1, sTemp1, sizeof(sTemp1), &iTempSz);

		rsTemp.sColumnEx(3, sTemp2, sizeof(sTemp2), &iTempSz);
		if(strcmpi(sTemp2, "D") == 0)
		{
			//Data File.
			strcpy(sLogicalData, sTemp1);
		}
		else if(strcmpi(sTemp2, "L") == 0)
		{
			//Log File.
			strcpy(sLogicalLog, sTemp1);
		}
	}

	rsTemp.Close();

	if(strlen(sLogicalData) > 0 && strlen(sLogicalLog) > 0)
	{
		sBaseSQL = "RESTORE DATABASE SQLExch_%s_IMP_Temp FROM DISK = '%s'\n"
			"WITH MOVE '%s' TO '%s\\SQLExch_%s_IMP_Temp_Dat.mdf',\n"
			"MOVE '%s' TO '%s\\SQLExch_%s_IMP_Temp_Log.ldf'\n";

		sprintf(sSQL, sBaseSQL, sDBName, sClientDBFileName,
			sLogicalData, gsSQLDataFiles, sDBName,
			sLogicalLog, gsSQLDataFiles, sDBName);
	}
	else{
		MsgBox("Failed to obtain a list of logical file names from the backup file specified.");
		return false;
	}

	return lpMySQL->ExecuteNonQuery(sSQL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateNewDB(CSQL *lpMySQL, char *sDBName)
{
	char sSQL[5120];
	SQLTEMPLATE MySQLTemp;

	memset(&MySQLTemp, 0, sizeof(MySQLTemp));

	MySQLTemp.iMask = SQLTMP_DBDIR | SQLTMP_DB;
	MySQLTemp.sDBDir = gsSQLDataFiles;
	MySQLTemp.sDB = sDBName;
	if(BuildSQLFromTemplate(gsPath, "SQLExch_Create_DB.sql", sSQL, &MySQLTemp))
	{
		return lpMySQL->ExecuteNonQuery(sSQL);
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateDatabaseIfNotExist(CSQL *lpMySQL, CSockSrvr *pSockSrvr, int iClient, const char *sDB)
{
	char sTemp[5120];
	CRecordSet rsExist;

	sprintf(sTemp, "SELECT Count(*) FROM SysDatabases WHERE Name = '%s'", sDB);
	CCI[iClient].cCustSQL.Execute(sTemp, &rsExist);

	if(!rsExist.Fetch())
	{
		WriteLog(pSockSrvr->icClientID[iClient], "Failed to fetch the SQL data.");
		giErrorCount++;
		return false;	
	}

	if(rsExist.lColumn(1) < 1)
	{
		sprintf(sTemp, "The cliet database [%s] does not exist. Creating it...", sDB);
		WriteLog(pSockSrvr->icClientID[iClient], sTemp);

		SQLTEMPLATE MySQLTemp;

		memset(&MySQLTemp, 0, sizeof(MySQLTemp));

		MySQLTemp.iMask = SQLTMP_DBDIR | SQLTMP_DB;
		MySQLTemp.sDBDir = gsSQLDataFiles;
		MySQLTemp.sDB = (char *) sDB;

		if(BuildSQLFromTemplate(gsPath, "SQLExch_Create_DB.sql", sTemp, &MySQLTemp))
		{
			if(!CCI[iClient].cCustSQL.ExecuteNonQuery(sTemp))
			{
				WriteLog(pSockSrvr->icClientID[iClient], "Failed to create the customer database.");
				giWarningCount++;
				return false;			
			}

			sprintf(sTemp, "UPDATE Companys SET Init = 1 WHERE CompName = '%s'", CCI[iClient].sCompanyName);
			CCI[iClient].cIndexSQL.ExecuteNonQuery(sTemp);
			CCI[iClient].bRequestInit = true;
		}
		else{
			WriteLog(pSockSrvr->icClientID[iClient], "Failed to obtain DB creation SQL.");
			giWarningCount++;
			return false;			
		}
	}

	rsExist.Close();
	
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CreateNewDBFromDBCreationScript(CSQL *lpMySQL, CSockSrvr *pSockSrvr, int iClient, char *sFileName)
{
	FILE *fSource = NULL;
	int iLen = 0;
	int iTables = 0;
	int iTable = 0;

	bool bThrowSQLErr = lpMySQL->bThrowErrors;

	char sTemp[1024];
	char sSQL[1024];
	char sDB[255];
	char sTable[255];
	char sPKName[255];
	char *sLargeSQL1 = NULL;
	//SQLTEMPLATE MySQLTemp;

	//memset(&MySQLTemp, 0, sizeof(MySQLTemp));

	if((fSource = fopen(sFileName, "rb")) == NULL)
	{
		return false;
	}

	sLargeSQL1 = (char *) calloc(sizeof(char), 1024 * 100);
	Assert(!sLargeSQL1, "Memory Allocation Error.");

	/*
	MySQLTemp.iMask = SQLTMP_DBDIR | SQLTMP_DB;
	MySQLTemp.sDBDir = gsSQLDataFiles;
	MySQLTemp.sDB = CCI[iClient].sCompanyDatabase;
	*/

	//----------------------------------------------------------------------------------------------------

	int iBytesRead = 0;

	while(true)
	{
		bool bFirstRecord = true;

		iTable = 0;

		//Read Table count.
		if(fread(&iTables, sizeof(iTables), 1, fSource) == 0)
		{
			break;
		}

		while(iTable < iTables && pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
		{
			//Read Database Name.
			fread(&iLen, sizeof(iLen), 1, fSource);
			fread(sDB, sizeof(char), iLen, fSource);
			sDB[iLen] = '\0';

			strcpy(sTemp, sDB);
			strcpy(sDB, "SQLExch_");
			strcat(sDB, CCI[iClient].sCompanyDBPrefix);
			strcat(sDB, "_");
			strcat(sDB, sTemp);
			
			if(bFirstRecord)
			{
				CreateDatabaseIfNotExist(lpMySQL, pSockSrvr, iClient, sDB);
				bFirstRecord = false;
			}

			//Read Table Name.
			fread(&iLen, sizeof(iLen), 1, fSource);
			fread(sTable, sizeof(char), iLen, fSource);
			sTable[iLen] = '\0';

			sprintf(sSQL, "Creating table: %s.", sTable);
			WriteLog(pSockSrvr->icClientID[iClient], sSQL);

			//Read PK Name.
			fread(&iLen, sizeof(iLen), 1, fSource);
			fread(sPKName, sizeof(char), iLen, fSource);
			sPKName[iLen] = '\0';

			//Read SQL.
			fread(&iLen, sizeof(iLen), 1, fSource);
			fread(sLargeSQL1, sizeof(char), iLen, fSource);
			sLargeSQL1[iLen] = '\0';

			//Drop the existing table.
			lpMySQL->bThrowErrors = false;
			sprintf(sSQL, "DROP TABLE [%s].[DBO].[%s]", sDB, sTable);
			lpMySQL->ExecuteNonQuery(sSQL);
			lpMySQL->bThrowErrors = true;

			ReplStr(sLargeSQL1, "{SQLExch_TAG_DB}", sDB);

			//Recreate the table.
			lpMySQL->ExecuteNonQuery(sLargeSQL1);

			iTable++;
		}

		if(pSockSrvr->bcConnected[iClient] == false || pSockSrvr->bcDisconnect[iClient] == true)
		{
			break;
		}

	}

	lpMySQL->bThrowErrors = bThrowSQLErr;

	fclose(fSource);
	free(sLargeSQL1);

	if(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
	{
		return true;
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
